@@ -226,8 +226,8 @@ totem_playlist_mrl_to_title (const gchar *mrl)
 
 	filename = g_path_get_basename (mrl);
 	unescaped = gnome_vfs_unescape_string_for_display (filename);
-
 	g_free (filename);
+
 	filename_for_display = g_filename_to_utf8 (unescaped,
 			-1,             /* length */
 			NULL,           /* bytes_read */
@@ -238,8 +238,12 @@ totem_playlist_mrl_to_title (const gchar *mrl)
 	{
 		filename_for_display = g_locale_to_utf8 (unescaped,
 				-1, NULL, NULL, NULL);
-		if (filename_for_display == NULL)
-			return unescaped;
+		if (filename_for_display == NULL) {
+			filename_for_display = g_filename_display_name
+				(unescaped);
+		}
+		g_free (unescaped);
+		return filename_for_display;
 	}
 
 	g_free (unescaped);
@@ -874,6 +878,7 @@ static void
 totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
 	GtkWidget *fs, *combo_box;
+	char *filename;
 	int response;
 
 	fs = gtk_file_chooser_dialog_new (_("Save Playlist"),
@@ -884,6 +889,11 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 			NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+	/* translators: Playlist is the default saved playlist filename,
+	 * without the suffix */
+	filename = g_strconcat (_("Playlist"), save_types[0].suffix, NULL);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (fs), filename);
+	g_free (filename);
 	combo_box = totem_playlist_save_add_format_combo_box (GTK_FILE_CHOOSER (fs));
 
 	if (playlist->_priv->save_path != NULL)
