@@ -429,17 +429,8 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	/* A 3rd into the file - multi-times = HACK for gst backend... */
-	i = 0;
-	do {
-		if (i != 0) {
-			g_usleep (HALF_SECOND);
-			g_main_iteration (FALSE);
-		}
-		length = bacon_video_widget_get_stream_length (bvw);
-		i++;
-	} while (length == 0 && i < 20);
-
+	/* Jump a 3rd into the file */
+	length = bacon_video_widget_get_stream_length (bvw);
 	if (length > MIN_LEN_FOR_SEEK)
 	{
 		if (bacon_video_widget_seek
@@ -461,16 +452,7 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	/* 10 seconds! */
-	i = 0;
-	do {
-		/* This is a hack for the gst backend because it resyncs
-		 * after a seekj asynchronously, and so a direct get_frame()
-		 * would still return the old frame. */
-		g_usleep (HALF_SECOND);
-		pixbuf = bacon_video_widget_get_current_frame (bvw);
-		i++;
-	} while (pixbuf == NULL && i < 20);
+	pixbuf = bacon_video_widget_get_current_frame (bvw);
 
 	/* Cleanup */
 	bacon_video_widget_close (bvw);
